/*
 * main.c
 *
 * tictactoe (SPiCboard)
 *
 * Created: 4/16/2018 2:11:35 PM
 * Author : rebeccca @ DESKTOP-1UREDUD
 */

// includes
#include "display.h"
#include "led.h"
#include "adc.h"
#include "button.h"
#include "avr/interrupt.h"
#include "timer.h"
#include <stdlib.h>

// variables
typedef enum {X, O, NONE} ENTRIES;

static volatile uint8_t buttonPressed = 0;
static ENTRIES entries [] = {NONE, NONE, NONE, NONE, NONE, NONE, NONE, NONE, NONE};


ISR(INT0_vect){
	buttonPressed = 1;
}
// functions
static void drawField(){

	sb_display_fillScreen(0);
	uint8_t bitmap [128*8];

	for( uint16_t i = 0; i < 128 * 8; ++i){
		bitmap[i] = 1 << 3;
	}

	sb_display_drawBitmap(2,15,1,80, bitmap);
	sb_display_drawBitmap(5,15,1,80, bitmap);

	for( uint16_t i = 0; i < 128 * 8; ++i){
		bitmap[i] = 0xff;
	}

	sb_display_drawBitmap(0,40,8,1, bitmap);
	sb_display_drawBitmap(0,70,8,1, bitmap);

}

static void drawSelected(uint8_t position){

	uint8_t bitmapX[28] = {0xf9, 0xf1, 0xe3, 0xc7, 0x8f, 0x1f, 0x3f, 0x3f, 0x1f, 0x8f, 0xc7, 0xe3, 0xf1, 0xf9,
							0x9f, 0x8f, 0xcf, 0xe3, 0xf1, 0xf8, 0xfc, 0xfc, 0xf8, 0xf1, 0xe3, 0xcf, 0x8f, 0x9f};
	uint8_t bitmapO[28] = {0x3f, 0x0f, 0x03, 0xc1, 0xf1, 0xf8, 0xfc, 0xfc, 0xf8, 0xf1, 0xc1, 0x03, 0x0f, 0x3f,
							0xfc, 0xf0, 0xc0, 0xc3, 0x8f, 0x1f, 0x3f, 0x3f, 0x1f, 0x8f, 0xc3, 0xc0, 0xf0, 0xfc};
	uint8_t bitmap [28] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
							0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff};

	uint8_t page = (position / 3) * 3;
	uint8_t col = (position % 3) * 30 + 18;

	if(entries[position] == X){
		sb_display_drawBitmap(page, col, 2, 14, bitmapX);
	}else if(entries[position] == O){
		sb_display_drawBitmap(page, col, 2, 14, bitmapO);
	}else{
		sb_display_drawBitmap(page, col, 2, 14, bitmap);
	}

}

static void removeSelected(uint8_t position){
	uint8_t page = (position / 3) * 3;
	uint8_t col = (position % 3) * 30 + 18;
	sb_display_drawBitmap(page, col, 2, 14, 0);


}

static void drawStatus(){
	uint8_t bitmapX[28] = {0x06, 0x0e, 0x1c, 0x38, 0x70, 0xe0, 0xc0, 0xc0, 0xe0, 0x70, 0x38, 0x1c, 0x0e, 0x06,
							0x60, 0x70, 0x38, 0x1c, 0x0e, 0x07, 0x03, 0x03, 0x07, 0x0e, 0x1c, 0x38, 0x70, 0x60};
	uint8_t bitmapO[28] = {0xc0, 0xf0, 0xfc, 0x3e, 0x0e, 0x07, 0x03, 0x03, 0x07, 0x0e, 0x3e, 0xfc, 0xf0, 0xc0,
							0x03, 0x0f, 0x3f, 0x3c, 0x70, 0xe0, 0xc0, 0xc0, 0xe0, 0x70, 0x3c, 0x3f, 0x0f, 0x03};

	for(uint8_t i = 0; i < 9; ++i){
		if(entries[i] == X){
			uint8_t page = (i/ 3 )* 3;
			uint8_t col = (i % 3) * 30 + 18;
			sb_display_drawBitmap(page, col, 2, 14, bitmapX);
		}else if(entries[i] == O){
			uint8_t page = (i/ 3 )* 3;
			uint8_t col = (i % 3) * 30 + 18;
			sb_display_drawBitmap(page, col, 2, 14, bitmapO);
		}

	}

}

static void select(ENTRIES current){
	int16_t poti;
	uint8_t free = 0;
	uint8_t pos = 0;
	uint8_t oldPos = 0;
	int8_t count;
	for (uint8_t i = 0; i < 9; ++i){
		if(entries[i] == NONE){
			++free;
		}
	}

	while(!buttonPressed){
		poti = sb_adc_read(POTI);
		count = (free * poti) / 1024 + 1;
		oldPos = pos;
		for(uint8_t i = 0; i < 9; ++i){
			if(entries[i] == NONE && count > 0){
				pos = i;
				--count;
			}
		}
		if(oldPos != pos){
			removeSelected(oldPos);
			if(entries[oldPos] != NONE){
				drawStatus();
			}
			drawSelected(pos);

		}
		if(free == 1){
			drawSelected(pos);
		}
	}
	cli();
	buttonPressed = 0;
	sei();
	entries[pos] = current;
	//current = current == X ? O : X;
	drawStatus(entries);
}

static uint8_t hasWon(ENTRIES type, ENTRIES e []){
	if(e[0] == type && e [1] == type && e[2] == type){
		return 1;
	}else if(e[3] == type && e [4] == type && e[5] == type){
		return 1;
	}else if(e[6] == type && e [7] == type && e[8] == type){
		return 1;
	}else if(e[0] == type && e [3] == type && e[6] == type){
		return 1;
	}else if(e[1] == type && e [4] == type && e[7] == type){
		return 1;
	}else if(e[2] == type && e [5] == type && e[8] == type){
		return 1;
	}else if(e[0] == type && e [4] == type && e[8] == type){
		return 1;
	}else if(e[2] == type && e [4] == type && e[6] == type){
		return 1;
	}else{
		return 0;
	}

}

//computer has X
static void calculateNew(){
	ENTRIES tmp [9];
	for (uint8_t i= 0; i < 9; ++i){
		if(entries[i] == NONE){
			for(uint8_t j= 0; j< 9; ++j){
				tmp[j] = entries[j];
			}
			tmp[i] = X;
			if(hasWon(X, tmp)){
				entries[i] = X;
				drawStatus();
				return;
			}
		}
	}

	for (uint8_t i= 0; i < 9; ++i){
		 if(entries[i] == NONE){
			 for(uint8_t j= 0; j< 9; ++j){
				 tmp[j] = entries[j];
			 }
			 tmp[i] = O;
			 if(hasWon(O, tmp)){
				entries[i] = X;
				drawStatus();
				return;
			 }
		 }
	}

	uint16_t r = rand();
	uint8_t free = 0;
	for(uint8_t i = 0; i < 9; ++i){
		if(entries[i] == NONE){
			++free;
		}
	}

	r = r % free;

	for(uint8_t i = 0; i < 9; ++i){
		if(entries[i] == NONE){
			if(r > 0){
				--r;
			}else{
				entries[i] = X;
				drawStatus();
				return;
			}
		}
	}


}

static uint8_t drawStartScreen(void){
	uint8_t bitmapTictactoe [174] = {0x60, 0x60, 0x60, 0xE0, 0xE0, 0xE0, 0x60, 0x60, 0x60, 0x00, //T oben
						0x00, 0x60, 0x60, 0x60, 0x00, // i oben
						0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // c oben
						0x60, 0x60, 0x60, 0xE0, 0xE0, 0xE0, 0x60, 0x60, 0x60, 0x00, //T oben
						0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // a oben
						0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // c oben
						0x60, 0x60, 0x60, 0xE0, 0xE0, 0xE0, 0x60, 0x60, 0x60, 0x00, //T oben
						0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // o oben
						0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // e oben
						0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, // T unten
					 	0x03, 0xFF, 0xFF, 0xFF, 0x00, // i unten
						0x00, 0x3C, 0x7E, 0xFF, 0xC3, 0xC3, 0xE7, 0x66, 0x24, 0x00, 0x00, 0x00, // c unten
						0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, // T unten
						0x70, 0xFB, 0xDB, 0xDB, 0xDB, 0xFF, 0xFF, 0xFE, 0x00,  // a unten
						0x00, 0x3C, 0x7E, 0xFF, 0xC3, 0xC3, 0xE7, 0x66, 0x24, 0x00, 0x00, 0x00,// c unten
						0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, // T unten
						0x3C, 0x7E, 0xFF, 0xC3, 0xC3, 0xFF, 0x7E, 0x3C, 0x00, // o unten
						0x00, 0x3C, 0x7E, 0xFF, 0xDB, 0xDB, 0xDF, 0x5E, 0x1C, 0x00 //e unten
					};
	uint8_t bitmapArrow [7] = {0x7F, 0x3E, 0x3E, 0x1C, 0x1C, 0x08, 0x08};
	uint8_t zeroBitmap [7] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};

	sb_display_fillScreen(0);
	sb_display_drawBitmap(0, 20, 2, 87, bitmapTictactoe);
	sb_display_showString(4, 15, "Single Player");
	sb_display_showString(6, 15, "Two Players");
	sb_display_drawBitmap(4, 6, 1, 7, bitmapArrow);

	int16_t poti;
	uint8_t oldSelected = 0;
	while(!buttonPressed){
		poti = sb_adc_read(POTI);
		if(poti < 512 && oldSelected != 0){
			sb_display_drawBitmap(4, 6, 1, 7, bitmapArrow);
			sb_display_drawBitmap(6, 6, 1, 7, zeroBitmap);
			oldSelected = 0;
		}else if (poti >= 512 && oldSelected != 1){
			sb_display_drawBitmap(6, 6, 1, 7, bitmapArrow);
			sb_display_drawBitmap(4, 6, 1, 7, zeroBitmap);
			oldSelected = 1;
		}
	}
	cli();
	buttonPressed = 0;
	sei();
	return oldSelected;
}

static void init(void){
	EIMSK |= (1 << INT0);
	EICRA &= ~(1 << ISC00);
	EICRA |= (1 << ISC01);

	DDRD &= ~(1 << PIND2);
	PORTD |= (1 << PIND2);

	sb_display_enable();

}

int main(void){
	srand((uint16_t) sb_adc_read(PHOTO));
	uint8_t user = 1;
	uint8_t modus;
	init();
	sei();
	uint8_t round = 0;
	if(sb_display_available() == 0){

		while(1){
			if(round == 0){
				modus = drawStartScreen();
				drawField();
				drawStatus();
			}
			if(user){
				sb_display_showStringSmall(3, 105, "0's");
				sb_display_showStringSmall(4, 105, "turn");
				select(O);
				user = 0;
			}else if(modus == 1){
				sb_display_showStringSmall(3, 105, "X's");
				sb_display_showStringSmall(4, 105, "turn");
				select(X);
				user = 1;
			}else{
				sb_display_showStringSmall(3, 105, "X's");
				sb_display_showStringSmall(4, 105, "turn");
				sb_timer_delay(500);
				calculateNew();
				user = 1;
			}
			++round;
			if(hasWon(X, entries)){
				sb_display_showStringWide(3, 20, "X has won!");
				sb_timer_delay(1800);
				for (uint8_t i = 0; i < 9; ++i){
					entries[i] = NONE;
				}
				drawField();
				drawStatus();
				round = 0;
				cli();
				buttonPressed = 0;
				sei();
			} else if(hasWon(O, entries)){
				sb_display_showStringWide(3, 20, "O has won!");
				sb_timer_delay(1800);
				for (uint8_t i = 0; i < 9; ++i){
					entries[i] = NONE;
				}
				drawField();
				drawStatus();
				round = 0;
				cli();
				buttonPressed = 0;
				sei();
			}
			if(round == 9){
				sb_display_showStringWide(3, 20, "Tied match!");
				sb_timer_delay(1800);
				for (uint8_t i = 0; i < 9; ++i){
					entries[i] = NONE;
				}
				drawField();
				drawStatus();
				round = 0;
				cli();
				buttonPressed = 0;
				sei();
			}

		}
	}else{
		sb_led_on(RED0);
	}
	while(1){

	}
}

